package it.eng.spago.jbi;

import it.eng.spago.dispatching.soapchannel.AdapterSOAP;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.DocumentFragment;

public class SpagoMessageReceiver implements Runnable {

	private DeliveryChannel mChannel = null;

	private boolean mProcess = true;

	private MessageExchange mExchange = null;

	private ComponentContext mContext = null;

	public SpagoMessageReceiver(DeliveryChannel channel,
			ComponentContext context) {
		mChannel = channel;
		mContext = context;
	}

	public void run() {

		mainLoop: while (mProcess) {
			try {
				Logger logger = mContext
						.getLogger(Constants.SPAGO_LOGGER, null);
				mExchange = mChannel.accept(5000);
				if (mExchange != null) {

					logger.log(Level.INFO, "Receveing message");
					if (mExchange instanceof InOut) {
						InOut inout = (InOut) mExchange;

						if (inout.getStatus() == ExchangeStatus.ACTIVE) {
							NormalizedMessage msg = inout.getInMessage();
							Source source = msg.getContent();

							logger.log(Level.INFO, "out message: "
									+ inout.getOutMessage());
							// create & set "out" message
							try {
								StringWriter sw = new StringWriter();
								Result output = new StreamResult(sw);
								Transformer transformer = TransformerFactory
										.newInstance().newTransformer();
								logger.log(Level.INFO, "Creato il transformer");
								transformer.transform(source, output);

								String serviceRequest = sw.toString();
								logger.log(Level.INFO, "serviceRequest :"
										+ serviceRequest);
								AdapterSOAP adapter = new AdapterSOAP();
								String serviceResponse = adapter
										.service(serviceRequest);
								logger.log(Level.INFO, "serviceResponse :"
										+ serviceResponse);

								NormalizedMessage responseMsg = inout
										.createMessage();
								logger.log(Level.INFO, "Creato il messaggio");
								responseMsg.setContent(new StreamSource(
										new StringReader(serviceResponse)));
								inout.setOutMessage(responseMsg);
								logger
										.log(Level.INFO,
												"Impostato il messaggio");

								mChannel.send(inout);
								logger.log(Level.INFO, "Spedito il messaggio");
							} catch (Exception e) {
								e.printStackTrace();
							}
						} else if (inout.getStatus() == ExchangeStatus.DONE) {
							// The consumer has received the response. The MEP
							// is done.
							logger.log(Level.INFO,
									"Ricevuto messaggio con status DONE");
						} else {
							// MEP status is ExchangeStatus.ERROR
							// any error recover (usually nothing) goes here
							logger.log(Level.INFO,
									"Ricevuto messaggio con status ERROR");
						}
					}
				}
			} catch (MessagingException me) {
				continue mainLoop;
			} catch (JBIException jex) {
				continue mainLoop;
			}
		}
	}

	public void stopProcessing() {
		this.mProcess = false;
	}

	public boolean isExchangeWithConsumerOkay(ServiceEndpoint se,
			MessageExchange me) {
		return true;
	}

	public boolean isExchangeWithProviderOkay(ServiceEndpoint se,
			MessageExchange me) {
		return true;
	}

	public ServiceEndpoint ResolveEndpointReference(DocumentFragment df) {
		return null;
	}

}
